@@ -285,6 +285,9 @@ g_vfs_backend_lockdownd_check (lockdownd_error_t cond, GVfsJob *job)
       g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
                         _("Permission denied"));
       break;
+    case LOCKDOWN_E_SSL_ERROR:
+      g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_CONNECTION_REFUSED,
+                        _("Unable to connect"));
     default:
       g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
                         _("Unhandled Lockdown error (%d)"), cond);
@@ -364,6 +367,28 @@ _idevice_event_cb (const idevice_event_t *event, void *user_data)
   exit (1);
 }
 
+static gboolean
+unpair_client (lockdownd_client_t client,
+               const char        *udid)
+{
+  lockdownd_error_t lerr;
+  gboolean ret = FALSE;
+
+  lerr = lockdownd_unpair (client, NULL);
+  if (lerr == LOCKDOWN_E_SUCCESS)
+    {
+      ret = TRUE;
+    }
+
+  return ret;
+}
+
+/* keep in sync with the choices array in g_vfs_backend_afc_mount() */
+enum {
+  CHOICE_TRY_AGAIN = 0,
+  CHOICE_CANCEL
+};
+
 /* Callback for mounting. */
 static void
 g_vfs_backend_afc_mount (GVfsBackend *backend,
@@ -382,12 +407,13 @@ g_vfs_backend_afc_mount (GVfsBackend *backend,
   int retries;
   idevice_error_t err;
   lockdownd_client_t lockdown_cli = NULL;
+  lockdownd_client_t lockdown_cli_old = NULL;
   char *camera_x_content_types[] = { "x-content/audio-player", "x-content/image-dcf", NULL};
   char *media_player_x_content_types[] = {"x-content/audio-player", NULL};
   char **dcim_afcinfo;
   plist_t value;
   lockdownd_error_t lerr;
-  const gchar *choices[] = {_("Try again"), _("Cancel")};
+  const gchar *choices[] = {_("Try again"), _("Cancel"), NULL}; /* keep in sync with the enum above */
   gboolean aborted = FALSE;
   gchar *message = NULL;
   gint choice;
@@ -478,7 +504,8 @@ g_vfs_backend_afc_mount (GVfsBackend *backend,
     goto out_destroy_service;
 
   /* first, connect without handshake to get preliminary information */
-  if (G_UNLIKELY(g_vfs_backend_lockdownd_check (lockdownd_client_new (self->dev, &lockdown_cli, "gvfsd-afc"), G_VFS_JOB(job))))
+  lerr = lockdownd_client_new (self->dev, &lockdown_cli, "gvfsd-afc");
+  if (G_UNLIKELY(g_vfs_backend_lockdownd_check (lerr, G_VFS_JOB(job))))
     goto out_destroy_dev;
 
   /* try to use pretty device name */
@@ -512,27 +539,32 @@ g_vfs_backend_afc_mount (GVfsBackend *backend,
 
   /* set correct freedesktop icon spec name depending on device model */
   value = NULL;
-  if (G_UNLIKELY(g_vfs_backend_lockdownd_check (lockdownd_get_value (lockdown_cli, NULL, "DeviceClass", &value), G_VFS_JOB(job))))
+  lerr = lockdownd_get_value (lockdown_cli, NULL, "DeviceClass", &value);
+  if (G_UNLIKELY(g_vfs_backend_lockdownd_check (lerr, G_VFS_JOB(job))))
     goto out_destroy_lockdown;
 
   plist_get_string_val (value, &self->model);
   if ((self->model != NULL) && (g_str_equal (self->model, "iPod") != FALSE))
     {
       g_vfs_backend_set_icon_name (G_VFS_BACKEND(self), "multimedia-player-apple-ipod-touch");
+      g_vfs_backend_set_symbolic_icon_name (G_VFS_BACKEND(self), "multimedia-player-apple-ipod-touch-symbolic");
     }
   else if ((self->model != NULL) && (g_str_equal (self->model, "iPad") != FALSE))
     {
       g_vfs_backend_set_icon_name (G_VFS_BACKEND(self), "computer-apple-ipad");
+      g_vfs_backend_set_symbolic_icon_name (G_VFS_BACKEND(self), "computer-apple-ipad-symbolic");
     }
   else
     {
       g_vfs_backend_set_icon_name (G_VFS_BACKEND(self), "phone-apple-iphone");
+      g_vfs_backend_set_symbolic_icon_name (G_VFS_BACKEND(self), "phone-apple-iphone-symbolic");
     }
 
   /* Get the major OS version */
   value = NULL;
   self->version = IOS_UNKNOWN;
-  if (G_LIKELY(g_vfs_backend_lockdownd_check (lockdownd_get_value (lockdown_cli, NULL, "ProductVersion", &value), G_VFS_JOB(job)) == 0))
+  lerr = lockdownd_get_value (lockdown_cli, NULL, "ProductVersion", &value);
+  if (G_LIKELY(g_vfs_backend_lockdownd_check (lerr, G_VFS_JOB(job)) == 0))
     {
       if (plist_get_node_type(value) == PLIST_STRING)
         {
@@ -568,14 +600,22 @@ g_vfs_backend_afc_mount (GVfsBackend *backend,
         }
     }
 
-  lockdownd_client_free (lockdown_cli);
+  /* save the old client until we connect with the handshake */
+  lockdown_cli_old = lockdown_cli;
   lockdown_cli = NULL;
 
   /* now, try to connect with handshake */
+  retries = 0;
   do {
     lerr = lockdownd_client_new_with_handshake (self->dev,
                                                 &lockdown_cli,
                                                 "gvfsd-afc");
+    if (lerr == LOCKDOWN_E_SSL_ERROR)
+      {
+        unpair_client (lockdown_cli_old, self->uuid);
+        continue;
+      }
+
     if (lerr != LOCKDOWN_E_PASSWORD_PROTECTED)
       break;
 
@@ -584,17 +624,20 @@ g_vfs_backend_afc_mount (GVfsBackend *backend,
       /* translators:
        * %s is the device name. 'Try again' is the caption of the button
        * shown in the dialog which is defined above. */
-      message = g_strdup_printf (_("The device '%s' is locked. Enter the passcode on the device and click 'Try again'."), display_name);
+      message = g_strdup_printf (_("The device “%s” is locked. Enter the passcode on the device and click “Try again”."), display_name);
 
     ret = g_mount_source_ask_question (src,
                                        message,
                                        choices,
-                                       2,
                                        &aborted,
                                        &choice);
-    if (!ret || aborted || (choice == 0))
+    if (!ret || aborted || (choice == CHOICE_CANCEL))
       break;
-  } while (1);
+  } while (retries++ < 10);
+
+  /* Now we're done with the old client */
+  lockdownd_client_free (lockdown_cli_old);
+  lockdown_cli_old = NULL;
 
   g_free (display_name);
   display_name = NULL;
@@ -605,9 +648,8 @@ g_vfs_backend_afc_mount (GVfsBackend *backend,
 
   switch (self->mode) {
     case ACCESS_MODE_AFC:
-      if (G_UNLIKELY(g_vfs_backend_lockdownd_check (lockdownd_start_service (lockdown_cli,
-                                                                             self->service, &port),
-                                                    G_VFS_JOB(job))))
+      lerr = lockdownd_start_service (lockdown_cli, self->service, &port);
+      if (G_UNLIKELY(g_vfs_backend_lockdownd_check (lerr, G_VFS_JOB(job))))
         {
           goto out_destroy_lockdown;
         }
@@ -619,9 +661,8 @@ g_vfs_backend_afc_mount (GVfsBackend *backend,
         }
       break;
     case ACCESS_MODE_HOUSE_ARREST:
-      if (G_UNLIKELY(g_vfs_backend_lockdownd_check (lockdownd_start_service (lockdown_cli,
-                                                                             "com.apple.mobile.installation_proxy", &port),
-                                                    G_VFS_JOB(job))))
+      lerr = lockdownd_start_service (lockdown_cli, "com.apple.mobile.installation_proxy", &port);
+      if (G_UNLIKELY(g_vfs_backend_lockdownd_check (lerr, G_VFS_JOB(job))))
         {
           g_warning ("couldn't start inst proxy");
           goto out_destroy_lockdown;
@@ -633,9 +674,8 @@ g_vfs_backend_afc_mount (GVfsBackend *backend,
           g_warning ("couldn't create inst proxy instance");
           goto out_destroy_lockdown;
         }
-      if (G_UNLIKELY(g_vfs_backend_lockdownd_check (lockdownd_start_service (lockdown_cli,
-                                                                             "com.apple.springboardservices", &port),
-                                                    G_VFS_JOB(job))))
+      lerr = lockdownd_start_service (lockdown_cli, "com.apple.springboardservices", &port);
+      if (G_UNLIKELY(g_vfs_backend_lockdownd_check (lerr, G_VFS_JOB(job))))
         {
           g_warning ("couldn't start SBServices proxy");
           goto out_destroy_lockdown;
@@ -1365,6 +1405,7 @@ g_vfs_backend_afc_set_info_from_afcinfo (GVfsBackendAfc *self,
 {
   GFileType type = G_FILE_TYPE_REGULAR;
   GIcon *icon = NULL;
+  GIcon *symbolic_icon = NULL;
   gchar *content_type = NULL;
   char *display_name;
   char *linktarget = NULL;
@@ -1457,11 +1498,13 @@ g_vfs_backend_afc_set_info_from_afcinfo (GVfsBackendAfc *self,
    * seems a little funny to put this in the backends.
    */
   if (g_file_attribute_matcher_matches (matcher, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE)
-      || g_file_attribute_matcher_matches (matcher, G_FILE_ATTRIBUTE_STANDARD_ICON))
+      || g_file_attribute_matcher_matches (matcher, G_FILE_ATTRIBUTE_STANDARD_ICON)
+      || g_file_attribute_matcher_matches (matcher, G_FILE_ATTRIBUTE_STANDARD_SYMBOLIC_ICON))
     {
       if (type == G_FILE_TYPE_DIRECTORY)
         {
           icon = g_themed_icon_new ("folder");
+          symbolic_icon = g_themed_icon_new ("folder-symbolic");
         }
       else
         {
@@ -1469,15 +1512,27 @@ g_vfs_backend_afc_set_info_from_afcinfo (GVfsBackendAfc *self,
             {
               icon = g_content_type_get_icon (content_type);
               if (G_IS_THEMED_ICON(icon))
-                g_themed_icon_append_name (G_THEMED_ICON(icon), "text-x-generic");
+                {
+                  g_themed_icon_append_name (G_THEMED_ICON(icon), "text-x-generic");
+                }
+
+              symbolic_icon = g_content_type_get_symbolic_icon (content_type);
+              if (G_IS_THEMED_ICON(symbolic_icon))
+                {
+                  g_themed_icon_append_name (G_THEMED_ICON(symbolic_icon), "text-x-generic-symbolic");
+                }
             }
         }
 
       if (icon == NULL)
         icon = g_themed_icon_new ("text-x-generic");
+      if (symbolic_icon == NULL)
+        symbolic_icon = g_themed_icon_new ("text-x-generic-symbolic");
 
       g_file_info_set_icon (info, icon);
+      g_file_info_set_symbolic_icon (info, symbolic_icon);
       g_object_unref (icon);
+      g_object_unref (symbolic_icon);
     }
 
   g_free (content_type);
@@ -1656,6 +1711,7 @@ g_vfs_backend_afc_set_info_from_app (GVfsBackendAfc *self,
                                      AppInfo *app_info)
 {
   GIcon *icon;
+  GIcon *symbolic_icon;
 
   /* content-type */
   g_file_info_set_content_type (info, "inode/directory");
@@ -1680,6 +1736,11 @@ g_vfs_backend_afc_set_info_from_app (GVfsBackendAfc *self,
       g_object_unref (icon);
     }
 
+  /* symbolic icon */
+  symbolic_icon = g_themed_icon_new ("folder-symbolic");
+  g_file_info_set_symbolic_icon (info, symbolic_icon);
+  g_object_unref (symbolic_icon);
+
   /* hidden ? */
   if (app_info && app_info->hidden)
     g_file_info_set_is_hidden (info, TRUE);
